;;;
;;;  SPDIF Buffer    ver 1.1
;;;                                               2004/3/14  UENO Tomohiro
;;;
;;;  for ATMEL AVRStudio 4.07
;;;
;;;  Device   Micro computer : ATMEL    AVR 90S8515 with external Xtal 10MHz

.include	"8515def.inc"



.def	tmp		= R16

.def	data_h		= R17
.def	data_l		= R18

.def	prev_clk	= R21
.def	current_clk	= R22

.def	c0x00		= R00
.def	c0x01		= R01
.def	c0xff		= R02

.equ	avr_lrckol	= 0	; portB
.equ	avr_lrckor	= 1	; portB
.equ	avr_lrcki	= 2	; portB
.equ	spoe		= 3	; portB

; data 15-08 = portD
; data 07-00 = portC

.equ	buf		= 0x100	; buf address = 0x100..0x1ff
.equ	bufsize		= (0x1ff-0x100+1)

	rjmp	reset	; 1 $000 reset vector

	reti		; 2 $001 INT0 

	reti		; 3 $002 INT1 

	reti		; 4 $003 TIMER1 CAPT

	reti		; 5 $004 TIMER1 COMPA 

	reti		; 6 $005 TIMER1 COMPB

	reti		; 7 $006 TIMER1 OVF1 

	reti		; 8 $007 TIMER1 OVF0 

	reti		; 9 $008 SPISTC

	reti		;10 $009 UART RX 

	reti		;11 $00A UART UDRE

	reti		;12 $00B UART TX

	reti		;13 $00C ANA_COMP



RESET:	
	ldi	tmp, high(RAMEND)

	out	SPH, tmp

	ldi	tmp, low(RAMEND)

	out	SPL,tmp

	; portA initialize

	ldi	tmp,0x00	; input (no use)

	out	DDRA,tmp

	ldi	tmp,0xff	; pullup

	out	PORTA,tmp



	; portB initialize

	ldi	tmp,0b00001000	; bit 4 (spoe) output , bit3-0 (avr_lrcki,avr_lrckol,avr_lrckor) input

	out	DDRB,tmp

	ldi	tmp,0b11110111	; pulup, spoe='0' (CPLD output 'Z')

	out	PORTB,tmp



	; portC initialize

	ldi	tmp,0xff	; output

	out	DDRC,tmp

	ldi	tmp,0xff	

	out	PORTC,tmp



	; portD initialize

	ldi	tmp,0xff	; output

	out	DDRD,tmp

	ldi	tmp,0xff	

	out	PORTD,tmp


	cbi	PORTB,spoe	; spoe=0 : AVR output = highZ
	ldi	tmp,0xff
	out	PORTC,tmp
	out 	PORTD,tmp

	ldi	tmp,0
	mov	c0x00,tmp
	ldi	tmp,1
	mov	c0x01,tmp
	ldi	tmp,0xff
	mov	c0xff,tmp

	ldi	XH,high(buf)
	ldi	XL,low(buf)
	ldi	data_h,0x00
	ldi	data_l,0x00
	ldi	tmp,128
	st	X+,data_h
	st	X+,data_l
	dec	tmp
	brne	PC-3
	
	ldi	data_l,100

	ldi	data_h,255

	ldi	tmp,255
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	dec	tmp
	brne	PC-8
	dec	data_h
	brne	PC-11
	dec	data_l
	brne	PC-14

	in	prev_clk,PINB
init:
	ldi	XH,high(buf)
	ldi	XL,low(buf)
	ldi	YH,high(buf)
	ldi	YL,low(buf+bufsize/2)

loop:
	cp	XL,YL
	breq	init

	in	current_clk,PINB
	bst	current_clk,avr_lrckol
	brts	out_l_ch
	bst	current_clk,avr_lrckor
	brts	out_r_ch

	bst	prev_clk,avr_lrcki
	brtc	PC+4
	bst	current_clk,avr_lrcki
	brtc	in_l_ch
	rjmp	loop
	bst	current_clk,avr_lrcki
	brts	in_r_ch
	rjmp	loop

in_r_ch:
	ldi	tmp,2
	or	XL,tmp
in_l_ch:
	out	PORTC,c0xff
	out	PORTD,c0xff
	out	DDRC,c0x00

	out	DDRD,c0x00

	ldi	tmp,0b11111111
	out	PORTB,tmp
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	in	data_h,PIND
	in	data_l,PINC
	ldi	tmp,0b11110111
	out	PORTB,tmp
	out	DDRC,c0xff

	out	DDRD,c0xff

	st	X+,data_h	
	st	X+,data_l
	ldi	XH,high(buf)
in_exit:
	mov	prev_clk,current_clk
	rjmp	loop

out_l_ch:
	ld	data_h,Y+	
	ld	data_l,Y+	
	out	PORTD,data_h
	out	PORTC,data_l
	ldi	YH,high(buf)
	sbic	PINB,avr_lrckol
	rjmp	PC-1
	rjmp	loop

out_r_ch:
	ldi	tmp,2
	or	YL,tmp
	ld	data_h,Y+	
	ld	data_l,Y+	
	out	PORTD,data_h
	out	PORTC,data_l
	ldi	YH,high(buf)
	sbic	PINB,avr_lrckor
	rjmp	PC-1
	rjmp	loop

